@@ -25,8 +25,8 @@
 #include "adapter.h"
 #include "device.h"
 
-#include <QtTest/QTest>
-#include <QtTest/QSignalSpy>
+#include <QTest>
+#include <QSignalSpy>
 #include <QDBusObjectPath>
 
 namespace BluezQt
@@ -39,6 +39,7 @@ using namespace BluezQt;
 void ManagerTest::initTestCase()
 {
     bluezqt_initFakeBluezTestRun();
+    FakeBluez::start(); // to check that it works
     Autotests::registerMetatypes();
 }
 
@@ -409,4 +410,52 @@ void ManagerTest::adapterWithDevicesRemovedTest()
     delete manager;
 }
 
+void ManagerTest::bug364416()
+{
+    // Bug 364416: Crash when device is added with adapter that is unknown to Manager
+
+    FakeBluez::start();
+    FakeBluez::runTest(QStringLiteral("bluez-standard"));
+
+    Manager *manager = new Manager;
+
+    InitManagerJob *job = manager->init();
+    job->exec();
+
+    QVERIFY(!job->error());
+
+    // Add device to invalid adapter
+    QVariantMap deviceProps;
+    deviceProps[QStringLiteral("Path")] = QVariant::fromValue(QDBusObjectPath("/org/bluez/hci0/dev_40_79_6A_0C_39_75"));
+    deviceProps[QStringLiteral("Adapter")] = QVariant::fromValue(QDBusObjectPath(QStringLiteral("/org/bluez/hci0")));
+    deviceProps[QStringLiteral("Address")] = QStringLiteral("40:79:6A:0C:39:75");
+    deviceProps[QStringLiteral("Name")] = QStringLiteral("TestDevice");
+    FakeBluez::runAction(QStringLiteral("devicemanager"), QStringLiteral("create-device"), deviceProps);
+
+    // Wait for Manager to receive the interfacesAdded signal
+    QTest::qWait(100);
+
+    delete manager;
+}
+
+void ManagerTest::bug377405()
+{
+    // Bug 377405: Property changes immediately after adapter is added are lost
+
+    FakeBluez::start();
+    FakeBluez::runTest(QStringLiteral("bluez-standard"));
+
+    Manager *manager = new Manager;
+
+    InitManagerJob *job = manager->init();
+    job->exec();
+
+    QVERIFY(!job->error());
+
+    FakeBluez::runAction(QStringLiteral("devicemanager"), QStringLiteral("bug377405"));
+
+    // Adapter property Powered is changed to true immediately after being added
+    QTRY_COMPARE(manager->isBluetoothOperational(), true);
+}
+
 QTEST_MAIN(ManagerTest)
